@@ -239,7 +239,7 @@ int use_black_magic(int mon)
 		}
 		else if (!zero_die(40))
 		{
-			/* we lack LOS, but passed the 1-in-80 chance to
+			/* we lack LOS, but passed the 1-in-40 chance to
 			 * close with the player by means of black magic. */
 			to_cast = MS_TELEPORT_ASSAULT;
 		}
@@ -458,14 +458,15 @@ int use_black_magic(int mon)
 		break;
 	case MS_FIRE_COLUMN:
 		mon_curses(mon);
-		print_msg("The fires of hell engulf you!\n");
+		print_msg("The fires of hell ");
 		if (player_resists_dtype(DT_FIRE))
 		{
-			print_msg("Even you are not immune to hellfire.\n");
+			print_msg("lightly singe you.\n");
 			damage_u(dice(1, 5), DEATH_KILLED_MON, permons[monsters[mon].mon_id].name);
 		}
 		else
 		{
+			print_msg("burn you!\n");
 			damage_u(dice(1, 20), DEATH_KILLED_MON, permons[monsters[mon].mon_id].name);
 		}
 		break;
@@ -480,7 +481,7 @@ void malignant_aura()
 
 void mon_curses(int mon)
 {
-	print_mon_name(3, monsters[mon].mon_id);
+	print_mon_name(monsters[mon].mon_id, 3);
 	print_msg(" points at you and curses horribly.\n");
 }
 
