@@ -46,7 +46,8 @@ enum game_cmd {
 	QUAFF_POTION, READ_SCROLL, WIELD_WEAPON, WEAR_ARMOUR,
 	TAKE_OFF_ARMOUR, PUT_ON_RING, REMOVE_RING, GIVE_HELP,
 	ATTACK, GET_ITEM, QUIT, GO_DOWN_STAIRS, STAND_STILL, EAT_FOOD,
-	DUMP_CHARA
+	DUMP_CHARA, INSPECT_ITEM, EXAMINE_MONSTER, RNG_TEST, SHOW_TERRAIN,
+	SHOW_DISCOVERIES
 };
 
 /* XXX enum terrain_num */
@@ -151,6 +152,16 @@ struct player {
 #define PM_DRAGON (PM_CENTAUR + 1)
 #define PM_REAL_COUNT ((PM_DRAGON) + 1)
 
+#define PMF_RESIST_FIRE	0x00000001
+#define PMF_RESIST_COLD	0x00000002
+#define PMF_RESIST_ELEC	0x00000004
+#define PMF_UNDEAD	0x00010000
+#define PMF_DEMONIC	0x00020000
+#define PMF_MAGICIAN	0x00040000
+#define PMF_ARCHER	0x00080000
+#define PMF_SMART	0x00100000
+#define PMF_STUPID	0x00200000
+
 struct permon {
 	const char name[48];
 	const char plural[48];
@@ -169,6 +180,7 @@ struct permon {
 	int defence;	/* Improved by OOD rating at 1:3. */
 	int exp;	/* Unaffected by OOD rating. */
 	int speed;	/* 0 = slow; 1 = normal; 2 = quick */
+	int flags;	/* resistances, AI settings, etc. */
 };
 
 /* XXX struct permobj */
@@ -217,15 +229,18 @@ struct permon {
 #define PO_RING_DOOM (PO_FIRST_RING + 5)
 #define PO_RING_TELEPORT (PO_FIRST_RING + 6)
 #define PO_LAST_RING PO_RING_TELEPORT
-#define PO_FOOD_RATION (PO_LAST_RING + 1)
-#define PO_FIRST_FOOD PO_FOOD_RATION
-#define PO_LAST_FOOD PO_FOOD_RATION
+#define PO_IRON_RATION (PO_LAST_RING + 1)
+#define PO_FIRST_FOOD PO_IRON_RATION
+#define PO_DRIED_FRUIT (PO_FIRST_FOOD + 1)
+#define PO_ELVEN_BREAD (PO_FIRST_FOOD + 2)
+#define PO_LAST_FOOD PO_ELVEN_BREAD
 #define PO_GOLD (PO_LAST_FOOD + 1)
 #define PO_REAL_COUNT ((PO_GOLD) + 1)
 
 struct permobj {
 	const char name[48];
 	const char plural[48];
+	const char description[160];
 	enum poclass_num poclass;
 	int rarity;	/* Chance in 100 of being thrown away and regen'd. */
 	int sym;
@@ -291,10 +306,14 @@ extern int select_dir(int *psy, int *psx);
 extern int getYN(const char *msg);
 extern int getyn(const char *msg);
 extern void press_enter(void);
+extern void pressanykey(void);
+extern void show_discoveries(void);
 
 /* "I've changed things that need to be redisplayed" flags. */
 extern int status_updated;
 extern int map_updated;
+/* "Show the player the terrain only" flag. */
+extern int show_terrain;
 
 /* XXX main.c data and funcs */
 extern int exclusive_flat(int lower, int upper); /* l+1 ... u-1 */
@@ -318,15 +337,18 @@ extern int get_room_y(int room);
 extern void room_reset(void);
 
 /* For now, I can't be arsed with a mapcell structure */
-#define DUN_SIZE 42
+#define DUN_WIDTH 42
+#define DUN_HEIGHT 42
+#define ROOM_HT_DELTA 4
+#define ROOM_WD_DELTA 4
 #define MAX_ROOMS 9
 
-extern int mapobject[DUN_SIZE][DUN_SIZE];
-extern int mapmonster[DUN_SIZE][DUN_SIZE];
-extern enum terrain_num terrain[DUN_SIZE][DUN_SIZE];
+extern int mapobject[DUN_HEIGHT][DUN_WIDTH];
+extern int mapmonster[DUN_HEIGHT][DUN_WIDTH];
+extern enum terrain_num terrain[DUN_HEIGHT][DUN_WIDTH];
 #define MAPFLAG_EXPLORED 0x00000001
-extern int mapflags[DUN_SIZE][DUN_SIZE];
-extern int roomnums[DUN_SIZE][DUN_SIZE];
+extern int mapflags[DUN_HEIGHT][DUN_WIDTH];
+extern int roomnums[DUN_HEIGHT][DUN_WIDTH];
 extern int roombounds[MAX_ROOMS][4];
 extern int roomlinkage[MAX_ROOMS][MAX_ROOMS];
 extern int depth;
@@ -338,6 +360,7 @@ extern const char *damtype_names[DT_COUNT];
 extern void flavours_init(void);
 extern void fprint_obj_name(FILE *fp, int obj);
 extern void print_obj_name(int obj);
+extern void describe_object(int obj);
 extern int create_obj(int po_idx, int quantity, int with_you, int y, int x);
 extern int drop_obj(int inv_idx);
 extern int create_obj_class(enum poclass_num pocl, int quantity, int with_you, int y, int x);
@@ -370,8 +393,10 @@ extern void compute_directions(int y1, int x1, int y2, int x2, int *pdy, int *pd
 extern void u_init(void);
 extern void write_char_dump(void);
 extern void do_death(enum death d, const char *what);
-extern void heal_u(int amount, int boost);
+extern void heal_u(int amount, int boost, int loud);
 extern int damage_u(int amount, enum death d, const char *what);
+extern int gain_body(int amount, int loud);
+extern int gain_agility(int amount, int loud);
 extern int drain_body(int amount, const char *what, int permanent);
 extern int drain_agility(int amount, const char *what, int permanent);
 extern void gain_experience(int amount);
