@@ -217,7 +217,7 @@ void death_drop(int mon)
 
 int mon_can_pass(int mon, int y, int x)
 {
-	if ((y < 0) || (x < 0) || (y >= DUN_SIZE) || (x >= DUN_SIZE))
+	if ((y < 0) || (x < 0) || (y >= DUN_HEIGHT) || (x >= DUN_WIDTH))
 	{
 		return 0;
 	}
@@ -291,11 +291,18 @@ void damage_mon(int mon, int amount, int by_you)
 		if (by_you)
 		{
 			print_msg("You kill ");
-			print_mon_name(mon, 1);
+			if (mon_visible(mon))
+			{
+				print_mon_name(mon, 1);
+			}
+			else
+			{
+				print_msg("something");
+			}
 			print_msg("!\n");
 			gain_experience(permons[mptr->mon_id].exp);
 		}
-		else
+		else if (mon_visible(mon))
 		{
 			print_mon_name(mon, 2);
 			print_msg(" dies.\n");
@@ -426,7 +433,7 @@ void summon_demon_near(int y, int x)
 	if ((terrain[y2][x2] == FLOOR) && (mapmonster[y2][x2] == -1) &&
 	    ((y2 != u.y) || (x2 != u.x)))
 	{
-		i = create_mon(PM_DEMON, y, x);
+		i = create_mon(PM_DEMON, y2, x2);
 		if (i != -1)
 		{
 			print_msg("Another demon appears!\n");
@@ -438,4 +445,57 @@ void summon_demon_near(int y, int x)
 	}
 }
 
+int mon_visible(int mon)
+{
+	int dy, dx;
+	if (monsters[mon].used == 0)
+	{
+		return 0;
+	}
+	dy = u.y - monsters[mon].y;
+	dx = u.x - monsters[mon].x;
+	if (((dy > -2) && (dy < 2) && (dx > -2) && (dx < 2)) ||
+	    ((roomnums[u.y][u.x] != -1) && (roomnums[u.y][u.x] == roomnums[monsters[mon].y][monsters[mon].x])))
+	{
+		return 1;
+	}
+	else
+	{
+		return 0;
+	}
+}
+
+void update_mon(int mon)
+{
+	int cansee;
+	if (monsters[mon].hpcur < monsters[mon].hpmax)
+	{
+		cansee = mon_visible(mon);
+		switch (monsters[mon].mon_id)
+		{
+		case PM_TROLL:
+			if (!(game_tick % 10))
+			{
+				if (cansee)
+				{
+					print_msg("The troll regenerates.\n");
+				}
+				heal_mon(mon, one_die(3) + 3, 0);
+			}
+			break;
+
+		case PM_ZOMBIE:
+			/* Zombies don't recover from their injuries. */
+			break;
+
+		default:
+			if (!(game_tick % 20))
+			{
+				heal_mon(mon, 1, cansee);
+			}
+			break;
+		}
+	}
+}
+
 /* monsters.c */
