@@ -183,19 +183,20 @@ static int consume_obj(int obj)
 int eat_food(int obj)
 {
 	struct obj *optr = objects + obj;
-	if (optr->obj_id != PO_FOOD_RATION)
+	if (permobjs[optr->obj_id].poclass != POCLASS_FOOD)
 	{
+		print_msg("Error: attempt to eat non-food (%d)!\n", optr->obj_id);
 		return -1;
 	}
 	if (u.food < 0)
 	{
-		print_msg("You ravenously devour the food ration!\n");
+		print_msg("You ravenously devour your food!\n");
 	}
 	else
 	{
 		print_msg("You eat some food.\n");
 	}
-	u.food += 2000;
+	u.food += 1500;
 	status_updated = 1;
 	display_update();
 	return consume_obj(obj);
@@ -207,15 +208,10 @@ int quaff_potion(int obj)
 	switch (optr->obj_id)
 	{
 	case PO_POT_BODY:
-		u.body++;
-		status_updated = 1;
-		print_msg("You feel stronger!\n");
+		gain_body(1, 1);
 		break;
 	case PO_POT_AGILITY:
-		u.agility++;
-		recalc_defence();
-		status_updated = 1;
-		print_msg("You feel more agile!\n");
+		gain_agility(1, 1);
 		break;
 	case PO_POT_WEAKNESS:
 		print_msg("You feel that was a bad idea!\n");
@@ -230,34 +226,35 @@ int quaff_potion(int obj)
 	case PO_POT_HEAL:
 		/* Heal player; if hit points brought to max, gain one
 		 * hit point. */
-		heal_u(dice(3, 12), 1);
+		heal_u(dice(3, 12), 1, 1);
 		break;
 	case PO_POT_RESTORATION:
 		print_msg("This potion makes you feel warm all over.\n");
+		status_updated = 1;
 		if (!zero_die(2))
 		{
 			if (u.adam)
 			{
-				print_msg("You feel less clumsy.\n");
 				u.adam = 0;
+				print_msg("You feel less clumsy.\n");
 			}
 			else if (u.bdam)
 			{
-				print_msg("You feel less feeble.\n");
 				u.bdam = 0;
+				print_msg("You feel less feeble.\n");
 			}
 		}
 		else
 		{
 			if (u.bdam)
 			{
-				print_msg("You feel less feeble.\n");
 				u.bdam = 0;
+				print_msg("You feel less feeble.\n");
 			}
 			else if (u.adam)
 			{
-				print_msg("You feel less clumsy.\n");
 				u.adam = 0;
+				print_msg("You feel less clumsy.\n");
 			}
 		}
 		break;
@@ -748,4 +745,48 @@ void damage_obj(int obj)
 	}
 } 
 
+void describe_object(int obj)
+{
+	struct obj *optr;
+	struct permobj *poptr;
+	print_obj_name(obj);
+	optr = objects + obj;
+	poptr = permobjs + optr->obj_id;
+	if (poptr->known)
+	{
+		print_msg("\n%s\n", poptr->description);
+	}
+	else
+	{
+		switch (poptr->poclass)
+		{
+		case POCLASS_NONE:
+		default:
+			print_msg("This unidentified permobj (%d) is indescribable.\n", optr->obj_id);
+			break;
+		case POCLASS_ARMOUR:
+			if ((optr->obj_id == PO_ROBE) ||
+			    (optr->obj_id == PO_ROBE_SHADOWS) ||
+			    (optr->obj_id == PO_ROBE_SWIFTNESS))
+			{
+				print_msg("\nA simple woolen robe.\n");
+			}
+			else
+			{
+				print_msg("\nAn unidentified and indescribable piece of armour (%d)\n", optr->obj_id);
+			}
+			break;
+		case POCLASS_SCROLL:
+			print_msg("\nA mysterious scroll.\nReading it will unleash its enchantment.\n");
+			break;
+		case POCLASS_POTION:
+			print_msg("\nA rather dubious-looking liquid.\nQuaffing it may be baleful or beneficial.\n");
+			break;
+		case POCLASS_RING:
+			print_msg("\nSome rings are baneful, some are beneficial, and\nsome are junk.\n");
+			break;
+		}
+	}
+}
+
 /* objects.c */
